@@ -1,15 +1,14 @@
 import math
+from functools import reduce
 from typing import List, Optional
 
 import regex as re
 import stanza
 from lang_trans.arabic import buckwalter
-from pelecanus import PelicanJson
 
 from .entity_mapper import EntityMapper
 from .fact import Fact
-from .parsers import AddressParser, ContactEmailNamePairParser, ContactEmailParser, ContactPhoneParserHighPrecision, \
-    ContactPhoneParserHighRecall, ContactPhoneParserStrict
+from .parsers import AddressParser, ContactEmailParser, ContactPhoneParserHighPrecision, ContactPhoneParserHighRecall, ContactPhoneParserStrict
 from .russian_transliterator import Transliterate
 
 
@@ -120,6 +119,36 @@ class Document:
             for fact in facts:
                 self.add_fact(fact)
 
+
+    @staticmethod
+    def get_text_from_nested_dict(document: dict, doc_path: str):
+        """
+        Get the last value of a dot delimited path from a nested dictionary.
+        :param document: Dictionary from which you want to parse a value.
+        :param doc_path: Dot delimited path to remove value from.
+        :return:
+        """
+        keys = doc_path.split(".")
+        return reduce(lambda d, key: d.get(key, None) if isinstance(d, dict) else None, keys, document)
+
+
+    @staticmethod
+    def set_text_to_nested_dict(document: dict, doc_path: str, value) -> dict:
+        """
+        :param document: Dictionary object to change.
+        :param doc_path: Dot delimited path to the key of a (potentially) nested dictionary,
+        :param value: Value you want to add to the last key of the doc_path.
+        :return: Modified dictionary.
+        """
+        keys = doc_path.split(".")
+        path_to_bottom_layer = keys[:-2]
+        for key in path_to_bottom_layer:
+            document = document.setdefault(key, {})
+        mlp_reserved_key = keys[-1]
+        document[mlp_reserved_key] = value
+        return document
+
+
     @staticmethod
     def remove_duplicates_with_ignored_keys(duplicates, ignore_keys):
         seen = set()
@@ -129,6 +158,7 @@ class Document:
                 yield duplicate
                 seen.add(index)
 
+
     @staticmethod
     def handle_null_values_in_facts(facts: List[dict]):
         container = []
@@ -148,6 +178,7 @@ class Document:
         else:
             return []
 
+
     def facts_to_json(self) -> dict:
         facts = [fact.to_json() for fact in self.__texta_facts]
         unique_facts = Document.remove_duplicate_facts(facts)
@@ -168,7 +199,7 @@ class Document:
             new_spans = []
             sent_index = 0
             for span in fact.spans:
-                span_len = span[1]-span[0]
+                span_len = span[1] - span[0]
                 sent_index = tokenized_text[:span[0]].count("\n")
                 # find last sent break before the match
                 matches = list(re.finditer(" \n ", tokenized_text[:span[0]]))
@@ -187,20 +218,6 @@ class Document:
                 fact.sent_index = sent_index
 
 
-    @staticmethod
-    def edit_doc(doc: dict, doc_path: str, new_value) -> dict:
-        """
-        :param doc: Original dictionary into which you add content.
-        :param doc_path: Dot separated field paths to the nested value you wish to add.
-        :param new_value: Value you wish to add into the nested path.
-        :return: Dictionary with the new value added into it according to the path.
-        """
-        wrapper = PelicanJson(doc)
-        path = doc_path.split(".")
-        wrapper.set_nested_value(path, new_value, force=True)
-        return wrapper.convert()
-
-
     @staticmethod
     def parse_doc(doc_path: str, document: dict) -> list:
         """
@@ -209,9 +226,7 @@ class Document:
         :param document: Document to be worked on.
         :return: List of text fields that will be processed by MLP.
         """
-        wrapper = PelicanJson(document)
-        doc_path_as_list = doc_path.split(".")
-        content = wrapper.safe_get_nested_value(doc_path_as_list, default=[])
+        content = Document.get_text_from_nested_dict(document, doc_path)
         if content and isinstance(content, str):
             return [content]
         # Check that content is non-empty list and there are only stings in the list.
@@ -230,13 +245,13 @@ class Document:
         """
         list_of_path_keys = self.doc_path.split(".")
         root_key = "{}_mlp".format(list_of_path_keys[-1])
-        path_to_mlp = list_of_path_keys[:-1] + [root_key] if len(list_of_path_keys) > 1 else [root_key]
+        path_to_mlp = f"{self.doc_path}.{root_key}"
         mlp_result = self.to_json(use_default_doc_path)
-        nested_dict_wrapper = PelicanJson(self.json_doc)
-        nested_dict_wrapper.set_nested_value(path_to_mlp, mlp_result["text_mlp"], force=True)
-        nested_dict_wrapper.set_nested_value(["texta_facts"], mlp_result["texta_facts"], force=True)
 
-        return nested_dict_wrapper.convert()
+        Document.set_text_to_nested_dict(document=self.json_doc, doc_path=path_to_mlp, value=mlp_result["text_mlp"])
+        Document.set_text_to_nested_dict(document=self.json_doc, doc_path="texta_facts", value=mlp_result["texta_facts"])
+
+        return self.json_doc
 
 
     def to_json(self, use_default_doc_path=True) -> dict:
@@ -306,12 +321,12 @@ class Document:
 
     def pos_tags(self):
         if "sentences" in self.analyzers:
-            for i,sent in enumerate(self.stanza_sentences):
+            for i, sent in enumerate(self.stanza_sentences):
                 tags_in_sent = [word.upos if word and word.upos and word.upos != "_" else "X" if word.upos == "_" else "X" for word in sent.words]
                 for tag in tags_in_sent:
                     self.__pos_tags.append(tag)
                 # if not last item
-                if i+1 < len(self.stanza_sentences):
+                if i + 1 < len(self.stanza_sentences):
                     self.__pos_tags.append("LBR")
         else:
             self.__pos_tags = [word.upos if word and word.upos and word.upos != "_" else "X" if word.upos == "_" else "X" for word in self.stanza_words]
@@ -323,12 +338,12 @@ class Document:
 
     def word_features(self):
         if "sentences" in self.analyzers:
-            for i,sent in enumerate(self.stanza_sentences):
+            for i, sent in enumerate(self.stanza_sentences):
                 tags_in_sent = [word.feats if word and word.feats and word.feats != "_" else "X" if word.feats == "_" else "X" for word in sent.words]
                 for tag in tags_in_sent:
                     self.__word_features.append(tag)
                 # if not last item
-                if i+1 < len(self.stanza_sentences):
+                if i + 1 < len(self.stanza_sentences):
                     self.__word_features.append("LBR")
         else:
             self.__word_features = [word.feats if word and word.feats and word.feats != "_" else "X" if word.feats == "_" else "X" for word in self.stanza_words]
@@ -419,13 +434,6 @@ class Document:
             (number.to_fact(Document.FACT_NAME_PHONE_STRICT, self.__get_doc_path("text")) for number in phone_numbers_strict))
 
 
-    def phone_high_recall(self):
-        text = self.get_words()
-        phone_numbers = ContactPhoneParserHighRecall(text, months=self.concat_resources["months"]).parse()
-        self.__texta_facts.extend(
-            (number.to_fact(Document.FACT_NAME_PHONE_HIGH_RECALL, self.__get_doc_path("text")) for number in phone_numbers))
-
-
     def phone_high_precision(self):
         text = self.get_words()
         phone_numbers_high_precision = ContactPhoneParserHighPrecision(text).parse()
