@@ -1,22 +1,27 @@
 # -*- coding: utf8 -*-
 import json
+from typing import List, Optional
+
 import regex as re
 
 
-class EntityMapper():
+class EntityMapper:
 
-    def __init__(self, entity_data):
+    def __init__(self, entity_data: dict):
+        """
+        :param entity_data: Dictionary where the key constitutes the label and the value a path to the JSON file containing the values.
+        """
         self.entities, self.longest_entity = self._load_entities(entity_data)
         self.entity_map = self._create_entity_map()
 
 
     def _create_entity_map(self):
-        '''
+        """
         Takes the entities and creates an entity_map
 
         Returns:
         entity_map -- A dictionary with entities as keys and type as the value of the key
-        '''
+        """
         entity_map = {}
         for entity_type, entity_list in self.entities.items():
             for entity in entity_list:
@@ -25,9 +30,9 @@ class EntityMapper():
         return entity_map
 
 
-    def _load_entities(self, entity_files):
-        '''
-        Loads entities from a file into a dict and stores the lenght of the longest entity.
+    def _load_entities(self, entity_files: dict):
+        """
+        Loads entities from a file into a dict and stores the length of the longest entity.
 
         Arguments:
         entity_data -- a list that contains the type and location of the entities, for example: [('addr', 'addresses.txt')]
@@ -36,7 +41,7 @@ class EntityMapper():
         entities -- A dict with the entity type and its contents  
 
         longest_entity -- the longest entity from all the entities
-        '''
+        """
 
         entities = {}
         longest_entity = 0
@@ -45,26 +50,26 @@ class EntityMapper():
             with open(entity_file, encoding='utf8') as fh:
                 try:
                     # Try to read as json
-                    for k, v in json.loads(fh.read()).items():
-                        v = [self._normalize_string(w) for w in v]
-                        if k not in entities:
-                            entities[k] = {}
+                    for label, values in json.loads(fh.read()).items():
+                        values = [self._normalize_string(value) for value in values]
+                        if label not in entities:
+                            entities[label] = {}
 
-                        for w in v:
-                            entities[k][w] = True
+                        for w in values:
+                            entities[label][w] = True
 
-                        entity_len = max([len(w.split(' ')) for w in v])
+                        entity_len = max([len(w.split(' ')) for w in values])
                         if entity_len > longest_entity:
                             longest_entity = entity_len
 
                 except:
                     # If fails, read as txt file
                     entities[entity_file] = {}
-                    for v in fh.readlines():
-                        v = self._normalize_string(v)
-                        entities[entity_file][v] = True
+                    for values in fh.readlines():
+                        values = self._normalize_string(values)
+                        entities[entity_file][values] = True
 
-                        entity_len = len(v.split(' '))
+                        entity_len = len(values.split(' '))
                         if entity_len > longest_entity:
                             longest_entity = entity_len
 
@@ -72,16 +77,16 @@ class EntityMapper():
 
 
     @staticmethod
-    def _normalize_string(entity):
-        '''
+    def _normalize_string(entity: str):
+        """
         Takes in an argument string 'entity' and returns it after lowering it and stripping whitespace
-        '''
+        """
         entity = entity.lower().strip()
         return entity
 
 
-    def map_entities(self, text, doc_path='', entity_types=None):
-        '''
+    def map_entities(self, text: str, doc_path='', entity_types: Optional[List] = None):
+        """
         Maps entities in given text.
 
         Arguments:
@@ -91,9 +96,9 @@ class EntityMapper():
 
 
         Returns:
-        filtered_mappings -- A sorted (descending by value lenght) dict that has redundant matches removed.
+        filtered_mappings -- A sorted (descending by value length) dict that has redundant matches removed.
         For example: {'addr': [{'span': (57, 69), 'doc_path': '', 'value': u'Street 2'}, {'span': (41, 51), 'doc_path': '', 'value': u'Street 1'}]}
-        '''
+        """
         if not entity_types:
             entity_types = list(self.entities.keys())
 
@@ -127,8 +132,8 @@ class EntityMapper():
 
 
     @staticmethod
-    def _remove_redundant_matches(mappings):
-        '''
+    def _remove_redundant_matches(mappings: dict):
+        """
         Takes in the mappings of entities and removes redundant matches.
 
         Arguments: 
@@ -136,14 +141,14 @@ class EntityMapper():
         For example: {'addr': [{'span': (41, 51), 'doc_path': '', 'value': u'Street 1'}, {'span': (57, 69), 'doc_path': '', 'value': u'Street 2'}]}
 
         Returns:
-        filtered_mappings -- A sorted (descending by value lenght) dict that has redundant matches removed.
+        filtered_mappings -- A sorted (descending by value length) dict that has redundant matches removed.
         For example: {'addr': [{'span': (57, 69), 'doc_path': '', 'value': u'Street 2'}, {'span': (41, 51), 'doc_path': '', 'value': u'Street 1'}]}
-        '''
+        """
         filtered_mappings = {}
 
         for entity_type in mappings.keys():
             filtered_matches = []
-            # Sorts by the lenght of the values in the entity type in descending order.
+            # Sorts by the length of the values in the entity type in descending order.
             sorted_matches = sorted(
                 mappings[entity_type], key=lambda k: len(k['value']), reverse=True)
 
@@ -169,8 +174,8 @@ class EntityMapper():
 
 
     @staticmethod
-    def _add_spans(mappings, text, doc_path):
-        '''
+    def _add_spans(mappings: dict, text: str, doc_path: str):
+        """
         Adds spans (character number locations of entities) and the document path to the entity dict.
 
         Arguments:
@@ -184,7 +189,7 @@ class EntityMapper():
         Returns:
         facts -- A dict that contains the facts about the entities, including the span, doc_path, type and value.
         For example: {'addr': [{'span': (41, 51), 'doc_path': '', 'value': u'Street 1'}, {'span': (57, 69), 'doc_path': '', 'value': u'Street 2'}]}
-        '''
+        """
         facts = {}
 
         for entity_type in mappings.keys():
@@ -206,8 +211,8 @@ class EntityMapper():
 
 
     @staticmethod
-    def _ngrams(inp, n):
-        '''
+    def _ngrams(inp: str, n: int):
+        """
         Takes in an input text and returns ngrams
 
         Arguments: 
@@ -219,7 +224,7 @@ class EntityMapper():
         returns a list that contains words as tuples, the word count depends on the argument 'n',
         for example if n is 1, the output will contain only one word per list value, ex: ([(u'example',), (u'string',), (u'here',)])
         but if n is 2, the output will contain two words per list value, ex: ([(u'example string',), (u'string here',))])
-        '''
+        """
         if type(inp) in [bytes, str]:
             inp = inp.split(' ')
         return zip(*[inp[i:] for i in range(n)])
