@@ -2,43 +2,32 @@ import logging
 import os
 import pathlib
 import shutil
-import torch
 from typing import List, Optional
 from urllib.parse import urlparse
 from urllib.request import urlopen
 
 import regex as re
 import stanza
+import torch
 from bs4 import BeautifulSoup
 from langdetect import detect
 
 from texta_mlp.document import Document
 from texta_mlp.entity_mapper import EntityMapper
-from texta_mlp.exceptions import LanguageNotSupported, CUDAException
-from texta_mlp.settings import (
-    DEFAULT_LANG_CODES,
-    DEFAULT_RESOURCE_DIR,
-    REFRESH_DATA,
-    CUSTOM_NER_MODEL_LANGS,
-    CUSTOM_NER_MODELS,
-    STANZA_NER_SUPPORT,
-    SUPPORTED_ANALYZERS,
-    DEFAULT_ANALYZERS,
-    ENTITY_MAPPER_DATA_URLS,
-    USE_GPU
-)
+from texta_mlp.exceptions import CUDAException, LanguageNotSupported
+from texta_mlp.settings import (CUSTOM_NER_MODELS, CUSTOM_NER_MODEL_LANGS, DEFAULT_ANALYZERS, DEFAULT_DOC_PATH_KEY, DEFAULT_LANG_CODES, DEFAULT_OUTPUT_KEY, DEFAULT_RESOURCE_DIR, ENTITY_MAPPER_DATA_URLS, META_KEY, REFRESH_DATA, STANZA_NER_SUPPORT, SUPPORTED_ANALYZERS, USE_GPU)
 
 
 class MLP:
 
     def __init__(
             self,
-            language_codes = DEFAULT_LANG_CODES,
-            default_language_code = DEFAULT_LANG_CODES[0],
+            language_codes=DEFAULT_LANG_CODES,
+            default_language_code=DEFAULT_LANG_CODES[0],
             use_default_language_code=True,
             resource_dir: str = DEFAULT_RESOURCE_DIR,
             ner_model_langs: list = CUSTOM_NER_MODEL_LANGS,
-            logging_level: str ="error",
+            logging_level: str = "error",
             use_gpu: bool = USE_GPU,
             gpu_device_id: int = 0,
             refresh_data: bool = REFRESH_DATA
@@ -66,7 +55,7 @@ class MLP:
             # select gpu based on env
             if gpu_device_id > 0:
                 device_count = torch.cuda.device_count()
-                if gpu_device_id > device_count-1:
+                if gpu_device_id > device_count - 1:
                     raise CUDAException(f"Invalid device id: {gpu_device_id}! Your machine only has {device_count} device(s).")
                 torch.cuda.set_device(gpu_device_id)
 
@@ -84,8 +73,9 @@ class MLP:
         self.download_stanza_resources(self.resource_dir, self.supported_langs, logger=self.logger)
         self.download_entity_mapper_resources(self.resource_dir, logger=self.logger)
 
+
     @staticmethod
-    def download_custom_ner_models(resource_dir: str, logger=None, custom_ner_model_urls: dict = CUSTOM_NER_MODELS, model_langs: list=None):
+    def download_custom_ner_models(resource_dir: str, logger=None, custom_ner_model_urls: dict = CUSTOM_NER_MODELS, model_langs: list = None):
         """
         Downloads custom ner models if not present in resources directory.
         """
@@ -102,6 +92,7 @@ class MLP:
                     with open(file_path, "wb") as fh:
                         fh.write(content)
 
+
     @staticmethod
     def download_stanza_resources(resource_dir: str, supported_langs: List[str], logger=None):
         """
@@ -177,7 +168,7 @@ class MLP:
         return lang
 
 
-    def generate_document(self, raw_text: str, analyzers: List[str], json_object: dict = None, doc_paths="text", lang=None, stanza_document=None):
+    def generate_document(self, raw_text: str, analyzers: List[str], json_object: dict = None, doc_paths=DEFAULT_DOC_PATH_KEY, lang=None, stanza_document=None):
         processed_text = MLP.normalize_input_text(raw_text)
         e = ""
 
@@ -257,16 +248,6 @@ class MLP:
         try:
             document = self.get_stanza_pipeline(lang)(raw_text)
             return document, e
-
-            for sentence in pipeline.sentences:
-                words = []
-                for word in sentence.words:
-                    words.append(word)
-                sentences.append(words)
-
-                for entity in sentence.entities:
-                    entities.append(entity)
-
         except KeyError as e:
             raise LanguageNotSupported(f"Language {lang} not supported. Check the list of supported languages.")
 
@@ -307,10 +288,12 @@ class MLP:
                 # document class.
                 self.__apply_analyzer(document, analyzer)
 
-            if "sentences" in analyzers and spans == "sentence":
+            if "sentences" in loaded_analyzers and spans == "sentence":
                 document.fact_spans_to_sent()
 
-            return document.to_json()
+            result = document.to_json()
+            result = self.__add_meta_to_mlp_output(result, DEFAULT_OUTPUT_KEY, loaded_analyzers, spans)
+            return result
         else:
             return None
 
@@ -332,6 +315,21 @@ class MLP:
             self.logger.exception(e)
 
 
+    def __add_meta_to_mlp_output(self, result: dict, field: str, analyzers: List[str], spans: str):
+        """Helper function to add meta information to an MLP document that has finished processing."""
+        if META_KEY not in result:
+            result[META_KEY] = {}
+
+        tokenization = "sentence" if "sentences" in analyzers else "text"
+        spans = "sentence" if "sentences" in analyzers and spans == "sentence" else "text"
+        result[META_KEY][field] = {
+            "tokenization": tokenization,
+            "spans": spans,
+            "analyzers": analyzers
+        }
+        return result
+
+
     def process_docs(self, docs: List[dict], doc_paths: List[str], analyzers=DEFAULT_ANALYZERS, spans: str = "text"):
         """
         :param docs: Contains tuples with two dicts inside them, the first being the document to be analyzed and the second is the meta information that corresponds to the document for transport purposes later on.
@@ -391,6 +389,7 @@ class MLP:
                             doc.fact_spans_to_sent()
 
                         result = doc.document_to_json(use_default_doc_path=False)
+                        result = self.__add_meta_to_mlp_output(result, doc_path, analyzers, spans)
                         docs[actual_index] = result
 
         return docs
